/*

gbmver.c - Shows the library version info of GBM (Generalized Bitmap Module).
           (On OS/2 the path to the detected GBM.DLL is shown as well.)

Author: Heiko Nitzsche

History
-------
29-Apr-2006: On OS/2 the DLL filename can now also be specified
             to be tested. This saves copying around gbmver.exe.

*/

#if defined(__OS2__) || defined(OS2)
#define INCL_DOS
#include <os2.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gbm.h"
#include "gbmtool.h"

/* ------------------------------ */

#if defined(__OS2__) || defined(OS2)

/**
 * Maximum filename length that will be parsed.
 * This does not yet include the \0 character !
 */
#if (defined(_MAX_DRIVE) && defined(_MAX_DIR) && defined(_MAX_FNAME) && defined(_MAX_EXT)) 
  #define GBMVER_FILENAME_MAX (_MAX_DRIVE+_MAX_DIR+_MAX_FNAME+_MAX_EXT)
#else
  #define GBMVER_FILENAME_MAX  800
#endif

/* ------------------------------ */

/** Show version info and found GBM.DLL on OS/2 platform. */
static BOOLEAN show_info_gbm_OS2(char * filename)
{
    int version = 0;

    const char * message = " Detected";

    PSZ      ModuleName                             = (filename == NULL) ? "GBM.DLL"  : filename;
    CHAR     FoundModuleName[GBMVER_FILENAME_MAX+1] = "";
    UCHAR    LoadError[GBMVER_FILENAME_MAX+1]       = "";
    HMODULE  ModuleHandle                           = NULLHANDLE;
    PFN      ModuleAddr                             = 0;
    APIRET   rc                                     = 0;

    rc = DosLoadModule(LoadError,
                       sizeof(LoadError)-1,
                       ModuleName,
                       &ModuleHandle);
    if (rc)
    {
       printf("\"%s\" not found.\n", ModuleName);
       return FALSE;
    }
 
    /* get the full path name */
    rc = DosQueryModuleName(ModuleHandle,
                            sizeof(FoundModuleName)-1,
                            FoundModuleName);
    if (rc)
    {
       printf("Could not extract module path name.\n");
       DosFreeModule(ModuleHandle);
       return FALSE;
    }

    /* call Gbm_init() */
    rc = DosQueryProcAddr(ModuleHandle,
                          0L,         
                          "Gbm_init",
                          &ModuleAddr);
    if (rc)
    {
       printf("%s \"%s\" has wrong file format.\n", message, ModuleName);
       DosFreeModule(ModuleHandle);
       return FALSE;
    }

    ModuleAddr();

    /* ------------------------ */

    /* call Gbm_version() */
    rc = DosQueryProcAddr(ModuleHandle,
                          0L,         
                          "Gbm_version",
                          &ModuleAddr);
    if (rc)
    {
       printf("%s \"%s\" has wrong file format.\n", message, ModuleName);
       DosFreeModule(ModuleHandle);
       return FALSE;
    }
 
    version = ModuleAddr();

    /* ------------------------ */

    /* call Gbm_deinit() */
    rc = DosQueryProcAddr(ModuleHandle,
                          0L,         
                          "Gbm_deinit",
                          &ModuleAddr);
    if (rc)
    {
       printf("%s \"%s\" has wrong file format.\n", message, ModuleName);
       DosFreeModule(ModuleHandle);
       return FALSE;
    }

    ModuleAddr();

    DosFreeModule(ModuleHandle);

    printf("%s \"%s\", version %d.%d\n", message, FoundModuleName, version/100, version-100);
    return TRUE;
}

/********************************************/

/** Show version info and found GBMDLG.DLL on OS/2 platform. */
static BOOLEAN show_info_gbmdlg_OS2(char * filename, char * filename_gbm)
{
    const char * message     = " Detected";

    PSZ      ModuleNameGbm   = (filename_gbm == NULL) ? "GBM.DLL"  : filename_gbm;
    HMODULE  ModuleHandleGbm = NULLHANDLE;

    PSZ      ModuleName                             = (filename == NULL) ? "GBMDLG.DLL"  : filename;
    CHAR     FoundModuleName[GBMVER_FILENAME_MAX+1] = "";
    UCHAR    LoadError[GBMVER_FILENAME_MAX+1]       = "";
    HMODULE  ModuleHandle                           = NULLHANDLE;
    PFN      ModuleAddr                             = 0;
    APIRET   rc                                     = 0;

    /* load GBM.DLL as a prerequisite for GBMDLG.DLL */
    rc = DosLoadModule(LoadError,
                       sizeof(LoadError)-1,
                       ModuleNameGbm,
                       &ModuleHandleGbm);
    if (rc)
    {
       printf("\"%s\" not found.\n", ModuleNameGbm);
       return FALSE;
    }

    /* load GBMDLG.DLL */
    rc = DosLoadModule(LoadError,
                       sizeof(LoadError)-1,
                       ModuleName,
                       &ModuleHandle);
    if (rc)
    {
       printf("\"%s\" not found.\n", ModuleName);
       DosFreeModule(ModuleHandleGbm);
       return FALSE;
    }
 
    /* get the full path name */
    rc = DosQueryModuleName(ModuleHandle,
                            sizeof(FoundModuleName)-1,
                            FoundModuleName);
    if (rc)
    {
       printf("Could not extract module path name.\n");
       DosFreeModule(ModuleHandleGbm);
       DosFreeModule(ModuleHandle);
       return FALSE;
    }

    /* check for GbmFileDlg() */
    rc = DosQueryProcAddr(ModuleHandle,
                          0L,         
                          "GbmFileDlg",
                          &ModuleAddr);
    if (rc)
    {
       printf("%s \"%s\" has wrong file format.\n", message, ModuleName);
       DosFreeModule(ModuleHandleGbm);
       DosFreeModule(ModuleHandle);
       return FALSE;
    }

    DosFreeModule(ModuleHandle);
    DosFreeModule(ModuleHandleGbm);

    printf("%s \"%s\"\n", message, FoundModuleName);
    return TRUE;
}

#else

/********************************************/

/** Show version info and found GBM.DLL on OS/2 platform. */
static BOOLEAN show_info_gbm_GENERIC(void)
{
    int version = 0;

    /* Standard handling for non-OS/2 operating systems */
    gbm_init();

    /* get library version */
    version = gbm_version();

    gbm_deinit();

    printf("GBM.DLL version: %d.%d\n", version/100, version-100);
    return TRUE;
}

#endif

/********************************************/
/********************************************/

int main(int argc, char * argv[])
{
#if defined(__OS2__) || defined(OS2)

    /* On OS/2 the user can provide a fully qualified
     * name of the GBM.DLL which is checked alternatively
     * to the automatic lookup.
     */
    char filename_gbm[GBMVER_FILENAME_MAX+1];
    char filename_gbmdlg[GBMVER_FILENAME_MAX+1];

    if (argc == 2)
    {
       /* check if the user specified a trailing \ */
       if (strlen(argv[1]) > GBMVER_FILENAME_MAX)
       {
          printf("Provided pathname is too long.\n");
          return 1;
       }

       if (argv[1][strlen(argv[1])-1] == '\\')
       {
         sprintf(filename_gbm   , "%sGBM.DLL"   , argv[1]);
         sprintf(filename_gbmdlg, "%sGBMDLG.DLL", argv[1]);
       }
       else
       {
         sprintf(filename_gbm   , "%s\\GBM.DLL"   , argv[1]);
         sprintf(filename_gbmdlg, "%s\\GBMDLG.DLL", argv[1]);
       }

       if (show_info_gbm_OS2(filename_gbm))
       {
          if (! show_info_gbmdlg_OS2(filename_gbmdlg, filename_gbm))
          {
             return 1;
          }
       }
       else
       {
          return 1;
       }
    }
    else
    {
       if (show_info_gbm_OS2(NULL))
       {
          if (! show_info_gbmdlg_OS2(NULL, NULL))
          {
             return 1;
          }
       }
       else
       {
          return 1;
       }
    }

#else

    if (! show_info_gbm_GENERIC())
    {
      return 1;
    }

#endif

    return 0;
}


